@@ -48,6 +48,7 @@
 
 static struct gvm_configuration config;
 static DBusConnection *dbus_connection;
+static char *gnome_mount = NULL;
 static LibHalContext *hal_ctx;
 
 struct _MountPolicy {
@@ -65,6 +66,10 @@ static GSList *mounted_volumes = NULL;
 static GHashTable *dialogs = NULL;
 
 
+static gboolean gvm_user_at_console (void);
+static gboolean gvm_user_active_at_console (void);
+
+
 typedef enum {
 	TYPE_BOOL,
 	TYPE_STRING
@@ -108,8 +113,12 @@ static struct {
 	{ GCONF_ROOT "autoburn_data_cd_command",  TYPE_STRING, &config.autoburn_data_cd_command  },
 	{ GCONF_ROOT "autoipod",                  TYPE_BOOL,   &config.autoipod                  },
 	{ GCONF_ROOT "autoipod_command",          TYPE_STRING, &config.autoipod_command          },
+	{ GCONF_ROOT "autokeyboard",              TYPE_BOOL,   &config.autokeyboard              },
+	{ GCONF_ROOT "autokeyboard_command",      TYPE_STRING, &config.autokeyboard_command      },
 	{ GCONF_ROOT "automount_drives",          TYPE_BOOL,   &config.automount_drives          },
 	{ GCONF_ROOT "automount_media",           TYPE_BOOL,   &config.automount_media           },
+	{ GCONF_ROOT "automouse",                 TYPE_BOOL,   &config.automouse                 },
+	{ GCONF_ROOT "automouse_command",         TYPE_STRING, &config.automouse_command         },
 	{ GCONF_ROOT "autophoto",                 TYPE_BOOL,   &config.autophoto                 },
 	{ GCONF_ROOT "autophoto_command",         TYPE_STRING, &config.autophoto_command         },
 	{ GCONF_ROOT "autopalmsync",              TYPE_BOOL,   &config.autopilot                 },
@@ -126,6 +135,8 @@ static struct {
 	{ GCONF_ROOT "autoprinter_command",       TYPE_STRING, &config.autoprinter_command       },
 	{ GCONF_ROOT "autorun",                   TYPE_BOOL,   &config.autorun                   },
 	{ GCONF_ROOT "autorun_path",              TYPE_STRING, &config.autorun_path              },
+	{ GCONF_ROOT "autotablet",                TYPE_BOOL,   &config.autotablet                },
+	{ GCONF_ROOT "autotablet_command",        TYPE_STRING, &config.autotablet_command        },
 };
 
 static GHashTable *gvm_settings_hash = NULL;
@@ -787,7 +798,8 @@ gvm_check_vcd (const char *udi, const char *device, const char *mount_point)
 static gboolean
 gvm_udi_is_storage_camera (const char *udi)
 {
-	char *storage_device = NULL, *physical_device = NULL;
+	char *physical_device = NULL;
+	char *storage_device = NULL;
 	gboolean is_camera = FALSE;
 	DBusError error;
 	
@@ -796,7 +808,7 @@ gvm_udi_is_storage_camera (const char *udi)
 		warn ("cannot get block.storage_device property: %s", error.message);
 		if (dbus_error_is_set (&error))
 			dbus_error_free (&error);
-		goto out;
+		return FALSE;
 	}
 	
 	if (!(physical_device = libhal_device_get_property_string (hal_ctx, storage_device, "storage.physical_device", &error))) {
@@ -874,6 +886,7 @@ gvm_udi_is_camera (const char *udi)
 	return is_camera;
 }
 
+#if 0
 /*
  * gvm_udi_is_ipod - checks if the udi is the mountable volume of an iPod
  *
@@ -882,8 +895,9 @@ gvm_udi_is_camera (const char *udi)
 static gboolean
 gvm_udi_is_ipod (const char *udi)
 {
-	char *product = NULL, *storage_device = NULL;
+	char *storage_device = NULL;
 	gboolean is_ipod = FALSE;
+	char *product = NULL;
 	DBusError error;
 	
 	dbus_error_init (&error);
@@ -891,7 +905,7 @@ gvm_udi_is_ipod (const char *udi)
 		warn ("cannot get block.storage_device property: %s", error.message);
 		if (dbus_error_is_set (&error))
 			dbus_error_free (&error);
-		goto out;
+		return FALSE;
 	}
 	
 	if (!(product = libhal_device_get_property_string (hal_ctx, storage_device, "info.product", &error))) {
@@ -913,6 +927,44 @@ gvm_udi_is_ipod (const char *udi)
 	
 	return is_ipod;
 }
+#endif
+
+/*
+ * gvm_udi_is_portable_media_player - checks if the udi is the mountable volume of a media player
+ *
+ * Returns TRUE if the device is a portable media player or FALSE otherwise.
+ */
+static gboolean
+gvm_udi_is_portable_media_player (const char *udi, gboolean *is_ipod)
+{
+	char *storage_device = NULL;
+	gboolean is_player = FALSE;
+	char *product = NULL;
+	DBusError error;
+	
+	if (is_ipod)
+		*is_ipod = FALSE;
+	
+	dbus_error_init (&error);
+	if (!(storage_device = libhal_device_get_property_string (hal_ctx, udi, "block.storage_device", &error))) {
+		warn ("cannot get block.storage_device property: %s", error.message);
+		if (dbus_error_is_set (&error))
+			dbus_error_free (&error);
+		return FALSE;
+	}
+	
+	if (is_ipod && (product = libhal_device_get_property_string (hal_ctx, storage_device, "info.product", NULL))) {
+		*is_ipod = !strcmp (product, "iPod");
+		libhal_free_string (product);
+	}
+	
+	if ((is_player = libhal_device_query_capability (hal_ctx, storage_device, "portable_audio_player", NULL)))
+		dbg ("%s detected: %s\n", is_ipod && *is_ipod ? "iPod" : "Generic music player", udi);
+	
+	libhal_free_string (storage_device);
+	
+	return is_player;
+}
 
 /*
  * gvm_run_camera - launch the camera application
@@ -950,16 +1002,15 @@ gvm_check_photos (const char *udi, const char *device, const char *mount_point)
 }
 
 /*
- * gvm_run_ipod - launch the ipod application
+ * gvm_run_portable_media_player - launch the ipod application
  */
 static void
-gvm_run_ipod (const char *udi, const char *device, const char *mount_point)
+gvm_run_portable_media_player (const char *udi, const char *device, const char *mount_point)
 {
 	if (config.autoipod_command != NULL)
 		gvm_run_command (config.autoipod_command, udi, device, mount_point);
 }
 
-
 /*
  * gvm_run_pilot - sync the pda palm pilot
  */
@@ -986,7 +1037,7 @@ gvm_run_pilot (const char *udi)
 }
 
 /*
- * gvm_run_pilot - sync the pda palm pilot
+ * gvm_run_pocketpc - sync the pda palm pilot
  */
 static void
 gvm_run_pocketpc (const char *udi)
@@ -1163,7 +1214,7 @@ ipod_photo_cb (GvmPromptCtx *ctx, int action)
 		gvm_run_camera (ctx->udi, ctx->device, ctx->path);
 		break;
 	case GVM_RESPONSE_SYNC_MUSIC:
-		gvm_run_ipod (ctx->udi, ctx->device, ctx->mount_point);
+		gvm_run_portable_media_player (ctx->udi, ctx->device, ctx->mount_point);
 		break;
 	default:
 		break;
@@ -1180,7 +1231,9 @@ ipod_photo_cb (GvmPromptCtx *ctx, int action)
 static void
 gvm_device_mounted (const char *udi)
 {
-	char *device = NULL, *mount_point = NULL;
+	char *mount_point = NULL;
+	char *device = NULL;
+	gboolean is_ipod = FALSE;
 	GvmPromptCtx *ctx;
 	DBusError error;
 	
@@ -1203,20 +1256,24 @@ gvm_device_mounted (const char *udi)
 	if (config.autophoto && gvm_udi_is_storage_camera (udi)) {
 		ctx = gvm_prompt_ctx_new (GVM_PROMPT_IMPORT_STORAGE_CAMERA, import_storage_camera_cb, udi, device, mount_point, NULL);
 		gvm_prompt (ctx, 0, NULL);
-	} else if (config.autoipod && gvm_udi_is_ipod (udi)) {
-		char *ipod_control;
-		
-		ipod_control = g_build_filename (mount_point, "iPod_Control", NULL);
-		
-		if (gvm_check_photos (udi, device, ipod_control)) {
-			/* we have ourselves an iPod Photo - need to prompt what to do */
-			ctx = gvm_prompt_ctx_new (GVM_PROMPT_IPOD_PHOTO, ipod_photo_cb, udi, device, mount_point, ipod_control);
-			gvm_prompt (ctx, 0, NULL);
+	} else if (config.autoipod && gvm_udi_is_portable_media_player (udi, &is_ipod)) {
+		if (is_ipod) {
+			char *ipod_control;
+			
+			ipod_control = g_build_filename (mount_point, "iPod_Control", NULL);
+			
+			if (gvm_check_photos (udi, device, ipod_control)) {
+				/* we have ourselves an iPod Photo - need to prompt what to do */
+				ctx = gvm_prompt_ctx_new (GVM_PROMPT_IPOD_PHOTO, ipod_photo_cb, udi, device, mount_point, ipod_control);
+				gvm_prompt (ctx, 0, NULL);
+			} else {
+				gvm_run_portable_media_player (udi, device, mount_point);
+			}
+			
+			g_free (ipod_control);
 		} else {
-			gvm_run_ipod (udi, device, mount_point);
+			gvm_run_portable_media_player (udi, device, mount_point);
 		}
-		
-		g_free (ipod_control);
 	} else {
 		gvm_autorun (udi, device, mount_point);
 	}
@@ -1242,57 +1299,84 @@ gvm_device_mount (const char *udi, int apply_policy)
 	dbus_uint64_t size;
 	int noptions = 0;
 	DBusError error;
+	gboolean retval;
+	char *command;
 	
 	dbg ("mounting %s...\n", udi);
 	
-	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
-						    "org.freedesktop.Hal.Device.Volume",
-						    "Mount"))) {
-		dbg ("mount failed for %s: could not create dbus message\n", udi);
-		return FALSE;
+	if (!gnome_mount || access (gnome_mount, F_OK | X_OK) != 0) {
+		g_free (gnome_mount);
+		gnome_mount = g_find_program_in_path ("gnome-mount");
 	}
 	
-	size = libhal_device_get_property_uint64 (hal_ctx, udi, "volume.size", NULL);
-	
-	fstype = "";
-	mount_point = "";
-	
-	if (size <= (256 * 1024 * 1024))
-		options[noptions++] = "sync";
-	options[noptions] = NULL;
-	poptions = options;
-	
-	if (!dbus_message_append_args (dmesg, DBUS_TYPE_STRING, &mount_point, DBUS_TYPE_STRING, &fstype,
-				       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &poptions, noptions,
-				       DBUS_TYPE_INVALID)) {
-		dbg ("mount failed for %s: could not append args to dbus message\n", udi);
+	if (gnome_mount != NULL) {
+		policy = g_new (struct _MountPolicy, 1);
+		policy->udi = g_strdup (udi);
+		policy->apply = apply_policy;
+		
+		g_hash_table_insert (mount_table, policy->udi, policy);
+		
+		command = g_strdup_printf ("%s --hal-udi=%%h", gnome_mount);
+		retval = gvm_run_command (command, udi, NULL, NULL);
+		g_free (command);
+		
+		if (!retval) {
+			g_hash_table_remove (mount_table, policy->udi);
+			g_free (policy->udi);
+			g_free (policy);
+		}
+		
+		return retval;
+	} else {
+		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+							    "org.freedesktop.Hal.Device.Volume",
+							    "Mount"))) {
+			dbg ("mount failed for %s: could not create dbus message\n", udi);
+			return FALSE;
+		}
+		
+		size = libhal_device_get_property_uint64 (hal_ctx, udi, "volume.size", NULL);
+		
+		fstype = "";
+		mount_point = "";
+		
+		if (size <= (256 * 1024 * 1024))
+			options[noptions++] = "sync";
+		options[noptions] = NULL;
+		poptions = options;
+		
+		if (!dbus_message_append_args (dmesg, DBUS_TYPE_STRING, &mount_point, DBUS_TYPE_STRING, &fstype,
+					       DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &poptions, noptions,
+					       DBUS_TYPE_INVALID)) {
+			dbg ("mount failed for %s: could not append args to dbus message\n", udi);
+			dbus_message_unref (dmesg);
+			return FALSE;
+		}
+		
+		policy = g_new (struct _MountPolicy, 1);
+		policy->udi = g_strdup (udi);
+		policy->apply = apply_policy;
+		
+		g_hash_table_insert (mount_table, policy->udi, policy);
+		
+		dbus_error_init (&error);
+		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+			dbg ("mount failed for %s: %s\n", udi, error.message);
+			g_hash_table_remove (mount_table, policy->udi);
+			dbus_message_unref (dmesg);
+			dbus_error_free (&error);
+			g_free (policy->udi);
+			g_free (policy);
+			return FALSE;
+		}
+		
+		dbg ("mount queued for %s\n", udi);
+		
 		dbus_message_unref (dmesg);
-		return FALSE;
+		dbus_message_unref (reply);
+		
+		return TRUE;
 	}
-	
-	policy = g_new (struct _MountPolicy, 1);
-	policy->udi = g_strdup (udi);
-	policy->apply = apply_policy;
-	
-	g_hash_table_insert (mount_table, policy->udi, policy);
-	
-	dbus_error_init (&error);
-	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
-		dbg ("mount failed for %s: %s\n", udi, error.message);
-		g_hash_table_remove (mount_table, policy->udi);
-		dbus_message_unref (dmesg);
-		dbus_error_free (&error);
-		g_free (policy->udi);
-		g_free (policy);
-		return FALSE;
-	}
-	
-	dbg ("mount queued for %s\n", udi);
-	
-	dbus_message_unref (dmesg);
-	dbus_message_unref (reply);
-	
-	return TRUE;
 }
 
 
@@ -1305,31 +1389,54 @@ static gboolean
 gvm_device_unmount (const char *udi)
 {
 	DBusMessage *dmesg, *reply;
+	char **options = NULL;
 	DBusError error;
+	gboolean retval;
+	char *command;
 	
 	dbg ("unmounting %s...\n", udi);
 	
-	if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
-						    "org.freedesktop.Hal.Device.Volume",
-						    "Unmount"))) {
-		dbg ("unmount failed for %s: could not create dbus message\n", udi);
-		return FALSE;
+	if (!gnome_mount || access (gnome_mount, F_OK | X_OK) != 0) {
+		g_free (gnome_mount);
+		gnome_mount = g_find_program_in_path ("gnome-mount");
 	}
 	
-	dbus_error_init (&error);
-	if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
-		dbg ("unmount failed for %s: %s\n", udi, error.message);
+	if (gnome_mount != NULL) {
+		command = g_strdup_printf ("%s --unmount --hal-udi=%%h", gnome_mount);
+		retval = gvm_run_command (command, udi, NULL, NULL);
+		g_free (command);
+		
+		return retval;
+	} else {
+		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", udi,
+							    "org.freedesktop.Hal.Device.Volume",
+							    "Unmount"))) {
+			dbg ("unmount failed for %s: could not create dbus message\n", udi);
+			return FALSE;
+		}
+		
+		if (!dbus_message_append_args (dmesg, DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options, 0,
+					       DBUS_TYPE_INVALID)) {
+			dbg ("unmount failed for %s: could not append args to dbus message\n", udi);
+			dbus_message_unref (dmesg);
+			return FALSE;
+		}
+		
+		dbus_error_init (&error);
+		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+			dbg ("unmount failed for %s: %s\n", udi, error.message);
+			dbus_message_unref (dmesg);
+			dbus_error_free (&error);
+			return FALSE;
+		}
+		
+		dbg ("unmount queued for %s\n", udi);
+		
 		dbus_message_unref (dmesg);
-		dbus_error_free (&error);
-		return FALSE;
+		dbus_message_unref (reply);
+		
+		return TRUE;
 	}
-	
-	dbg ("unmount queued for %s\n", udi);
-	
-	dbus_message_unref (dmesg);
-	dbus_message_unref (reply);
-	
-	return TRUE;
 }
 
 /*
@@ -1701,8 +1808,9 @@ camera_device_added (const char *udi, const char *capability __attribute__((__un
 {
 	GvmPromptCtx *ctx;
 	
+	/* check that the camera is a non-storage camera */
 	if (!gvm_udi_is_camera (udi))
-		return FALSE;
+		return TRUE;
 	
 	if (!(config.autophoto && config.autophoto_command))
 		return TRUE;
@@ -1729,7 +1837,6 @@ input_device_added (const char *udi, const char *capability)
 	/* input device (keyboard, mouse, wacom tablet, etc...) */
 	const char *command = NULL;
 	int autoexec = FALSE;
-	int handled = FALSE;
 	DBusError error;
 	char *device;
 	size_t i;
@@ -1742,8 +1849,12 @@ input_device_added (const char *udi, const char *capability)
 		}
 	}
 	
+	if (i == G_N_ELEMENTS (inputs)) {
+		/* we don't handle this type of device */
+		return FALSE;
+	}
+	
 	if (autoexec && command) {
-		handled = TRUE;
 		dbus_error_init (&error);
 		if ((device = libhal_device_get_property_string (hal_ctx, udi, "input.device", &error))) {
 			gvm_run_command (command, udi, device, NULL);
@@ -1754,7 +1865,7 @@ input_device_added (const char *udi, const char *capability)
 		}
 	}
 	
-	return handled;
+	return TRUE;
 }
 
 static gboolean
@@ -1765,21 +1876,50 @@ pda_device_added (const char *udi, const char *capability __attribute__((__unuse
 	
 	dbus_error_init (&error);
 	
-	if ((platform = libhal_device_get_property_string (hal_ctx, udi, "pda.platform", &error))) {
-		if (!strcmp (platform, "palm")) {
-			if (config.autopilot)
-				gvm_run_pilot (udi);
-		} else if (!strcmp (platform, "pocketpc")) {
-			if (config.autopocketpc)
-				gvm_run_pocketpc (udi);
-		}
-		
-		libhal_free_string (platform);
-	} else {
+	if (!(platform = libhal_device_get_property_string (hal_ctx, udi, "pda.platform", &error))) {
 		warn ("cannot get pda.platform property: %s", error.message);
 		dbus_error_free (&error);
+		return TRUE;
 	}
 	
+	if (!strcmp (platform, "palm")) {
+		if (config.autopilot)
+			gvm_run_pilot (udi);
+	} else if (!strcmp (platform, "pocketpc")) {
+		if (config.autopocketpc)
+			gvm_run_pocketpc (udi);
+	}
+	
+	libhal_free_string (platform);
+	
+	return TRUE;
+}
+
+static gboolean
+media_player_device_added (const char *udi, const char *capability __attribute__((__unused__)))
+{
+	char *access_method;
+	DBusError error;
+	
+	dbus_error_init (&error);
+	
+	if (!(access_method = libhal_device_get_property_string (hal_ctx, udi, "portable_audio_player.access_method", &error))) {
+		warn ("cannot get portable_audio_player.access_method property: %s", error.message);
+		dbus_error_free (&error);
+		return TRUE;
+	}
+	
+	if (!strcmp (access_method, "storage")) {
+		/* these get handled after being mounted */
+		g_free (access_method);
+		return TRUE;
+	}
+	
+	g_free (access_method);
+	
+	if (config.autoipod)
+		gvm_run_portable_media_player (udi, NULL, NULL);
+	
 	return TRUE;
 }
 
@@ -1797,14 +1937,15 @@ static struct {
 	const char *capability;
 	DeviceAddedHandler handler;
 } devices[] = {
-	{ "block",          block_device_added   },
-	{ "camera",         camera_device_added  },
-	/*{ "input",          input_device_added   },*/ /* we don't handle generic input devices (yet?) */
-	{ "input.keyboard", input_device_added   },
-	{ "input.mouse",    input_device_added   },
-	{ "input.tablet",   input_device_added   },
-	{ "pda",            pda_device_added     },
-	{ "printer",        printer_device_added },
+	{ "block",                 block_device_added        },
+	{ "camera",                camera_device_added       },
+	/*{ "input",                 input_device_added        },*/ /* we don't handle generic input devices (yet?) */
+	{ "input.keyboard",        input_device_added        },
+	{ "input.mouse",           input_device_added        },
+	{ "input.tablet",          input_device_added        },
+	{ "pda",                   pda_device_added          },
+	{ "portable_audio_player", media_player_device_added },
+	{ "printer",               printer_device_added      },
 };
 
 /** Invoked when a device is added to the Global Device List. 
@@ -1819,6 +1960,9 @@ hal_device_added (LibHalContext *ctx __attribute__((__unused__)),
 	char **capabilities;
 	size_t i, j, n;
 	
+	if (!gvm_user_active_at_console ())
+		return;
+	
 	dbg ("Device added: %s\n", udi);
 	
 	if (!(capabilities = libhal_device_get_property_strlist (hal_ctx, udi, "info.capabilities", NULL)))
@@ -1925,7 +2069,7 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 			
 			g_free (policy->udi);
 			g_free (policy);
-		} else {
+		} else if (gvm_user_active_at_console ()) {
 			dbg ("not in mount queue: %s\n", udi);
 			gvm_device_mounted (udi);
 		}
@@ -1945,6 +2089,84 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
 	}
 }
 
+static void
+gvm_device_eject (const char *udi)
+{
+	char *storage, **volumes = NULL;
+	DBusMessage *dmesg, *reply;
+	const char *volume = udi;
+	char **options = NULL;
+	DBusError error;
+	char *command;
+	int i, n;
+	
+	if (!libhal_device_get_property_bool (hal_ctx, udi, "block.is_volume", NULL)) {
+		dbus_error_init (&error);
+		volumes = libhal_find_device_by_capability (hal_ctx, "volume", &n, &error);
+		if (dbus_error_is_set (&error)) {
+			dbus_error_free (&error);
+			return;
+		}
+		
+		volume = NULL;
+		for (i = 0; i < n; i++) {
+			volume = volumes[i];
+			
+			if (!(storage = libhal_device_get_property_string (hal_ctx, volume, "info.parent", NULL)))
+				continue;
+			
+			if (!strcmp (udi, storage)) {
+				libhal_free_string (storage);
+				break;
+			}
+			
+			libhal_free_string (storage);
+		}
+	}
+	
+	dbg ("ejecting %s...\n", volume);
+	
+	if (!gnome_mount || access (gnome_mount, F_OK | X_OK) != 0) {
+		g_free (gnome_mount);
+		gnome_mount = g_find_program_in_path ("gnome-mount");
+	}
+	
+	if (gnome_mount != NULL) {
+		command = g_strdup_printf ("%s --eject --hal-udi=%%h", gnome_mount);
+		gvm_run_command (command, volume, NULL, NULL);
+		g_free (command);
+	} else {
+		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", volume,
+							    "org.freedesktop.Hal.Device.Volume",
+							    "Eject"))) {
+			goto done;
+		}
+		
+		if (!dbus_message_append_args (dmesg, DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &options, 0,
+					       DBUS_TYPE_INVALID)) {
+			dbg ("eject failed for %s: could not append args to dbus message\n", udi);
+			dbus_message_unref (dmesg);
+			goto done;
+		}
+		
+		dbus_error_init (&error);
+		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
+			dbg ("eject failed for %s: %s\n", volume, error.message);
+			dbus_message_unref (dmesg);
+			dbus_error_free (&error);
+			goto done;
+		}
+		
+		dbus_message_unref (dmesg);
+		dbus_message_unref (reply);
+	}
+	
+ done:
+	
+	if (volumes)
+		libhal_free_string_array (volumes);
+}
+
 /** Invoked when a device in the GDL emits a condition that cannot be
  *  expressed in a property (like when the processor is overheating)
  *
@@ -1954,68 +2176,15 @@ hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
  *  @param  message             D-BUS message with parameters
  */
 static void
-hal_device_condition (LibHalContext *ctx, const char *udi, const char *condition_name,
+hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
+		      const char *udi, const char *condition_name,
 		      const char *condition_details __attribute__((__unused__)))
 {
-	if (!strcmp (condition_name, "EjectPressed")) {
-		char *storage, **volumes = NULL;
-		DBusMessage *dmesg, *reply;
-		const char *volume = udi;
-		DBusError error;
-		int i, n;
-		
-		if (!libhal_device_get_property_bool (ctx, udi, "block.is_volume", NULL)) {
-			dbus_error_init (&error);
-			volumes = libhal_find_device_by_capability (ctx, "volume", &n, &error);
-			if (dbus_error_is_set (&error)) {
-				dbus_error_free (&error);
-				return;
-			}
-			
-			volume = NULL;
-			for (i = 0; i < n; i++) {
-				volume = volumes[i];
-				
-				if (!(storage = libhal_device_get_property_string (ctx, volume, "info.parent", NULL)))
-					continue;
-				
-				if (!strcmp (udi, storage)) {
-					libhal_free_string (storage);
-					break;
-				}
-				
-				libhal_free_string (storage);
-			}
-		}
-		
-		dbg ("ejecting %s...\n", volume);
-		
-		if (!(dmesg = dbus_message_new_method_call ("org.freedesktop.Hal", volume,
-							    "org.freedesktop.Hal.Device.Volume",
-							    "Eject"))) {
-			if (volumes)
-				libhal_free_string_array (volumes);
-			return;
-		}
-		
-		dbus_error_init (&error);
-		if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error))) {
-			dbg ("eject failed for %s: %s\n", volume, error.message);
-			dbus_message_unref (dmesg);
-			dbus_error_free (&error);
-			
-			if (volumes)
-				libhal_free_string_array (volumes);
-			
-			return;
-		}
-		
-		if (volumes)
-			libhal_free_string_array (volumes);
-		
-		dbus_message_unref (dmesg);
-		dbus_message_unref (reply);
-	}
+	if (!gvm_user_active_at_console ())
+		return;
+	
+	if (!strcmp (condition_name, "EjectPressed"))
+		gvm_device_eject (udi);
 }
 
 /** Integrate a dbus mainloop. 
@@ -2155,7 +2324,6 @@ mount_all (LibHalContext *ctx)
 		if (!(drive = libhal_device_get_property_string (ctx, udi, "info.parent", NULL)))
 			continue;
 		
-		mount = FALSE;
 		if (libhal_device_property_exists (ctx, drive, "storage.hotpluggable", NULL)
 		    && libhal_device_get_property_bool (ctx, drive, "storage.hotpluggable", NULL))
 			mount = config.automount_drives;
@@ -2163,7 +2331,7 @@ mount_all (LibHalContext *ctx)
 			 && libhal_device_get_property_bool (ctx, drive, "storage.removable", NULL))
 			mount = config.automount_media;
 		else
-			mount = TRUE;
+			mount = gvm_automount_enabled (udi);
 		libhal_free_string (drive);
 		
 		if (!mount)
@@ -2259,59 +2427,101 @@ gvm_die (GnomeClient *client __attribute__((__unused__)),
 	gtk_main_quit ();
 }
 
-/* assume that display :0 is the system console */
-static int
-gvm_running_on_console (void)
+
+/* checks that the user is logged-in at the console (which does not necessarily infer an *active* session) */
+static gboolean
+gvm_user_at_console (void)
 {
-	const char *str;
+	static unsigned int exists = 0;
+	char *path;
 	
-	if (!(str = DisplayString (GDK_DISPLAY ())))
-		return TRUE;
+	if (!(exists & 0x2)) {
+		path = g_build_filename (GVM_CONSOLE_AUTH_DIR, g_get_user_name (), NULL);
+		exists = (access (path, F_OK) == 0) ? 0x3 : 0x2;
+		g_free (path);
+	}
 	
-	if (!(str = strchr (str, ':')))
-		return TRUE;
-	
-	str++;
-	if (str[0] == ':')
-		str++;
-	
-	return strtoul (str, NULL, 10) == 0;
+	return (exists & 0x1);
 }
 
+/* checks that the user's session is the active session at the console */
+static gboolean
+gvm_user_active_at_console (void)
+{
+	/* FIXME: it is currently impossible to get this information */
+	return TRUE;
+}
+
+
+#ifndef POPT_TABLEEND
+#define POPT_TABLEEND { NULL, '\0', 0, NULL, '\0', NULL, NULL }
+#endif
+
+static const struct poptOption options[] = {
+	{ "version", 'v', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 'v',
+	  N_("Print version and exit"), NULL },
+	{ "daemon", 'd', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, NULL, 'd',
+	  N_("Run as a daemon"), "<yes|no>" },
+	{ "no-daemon", 'n', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 'n',
+	  N_("Don't run as a daemon"), NULL },
+	{ "secret-mode", 's', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH, NULL, 's',
+	  N_("Run in secret mode"), NULL },
+	POPT_TABLEEND
+};
+
 int
 main (int argc, char *argv[])
 {
-	GnomeClient *client;
 	gboolean daemonize = TRUE;
-
-	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-			    argc, argv, GNOME_PARAM_NONE);
-
-	/* We only have one option, so this is fine for now ... */
-	if (argc >= 2 && !strcmp (argv[1], "--no-daemon"))
-		daemonize = FALSE;
-
+	GnomeProgram *program;
+	GnomeClient *client;
+	poptContext ctx;
+	const char *arg;
+	int rc;
+	
+	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+	textdomain (PACKAGE);
+	
+	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+				      GNOME_PARAM_POPT_FLAGS, POPT_CONTEXT_POSIXMEHARDER,
+				      GNOME_PARAM_POPT_TABLE, options, GNOME_PARAM_NONE);
+	
+	g_object_get ((GObject *) program, GNOME_PARAM_POPT_CONTEXT, &ctx, NULL);
+	
+	poptResetContext (ctx);
+	
+	while ((rc = poptGetNextOpt (ctx)) > 0) {
+		switch (rc) {
+		case 'v':
+			fprintf (stdout, "%s version %s\n", PACKAGE, VERSION);
+			poptFreeContext (ctx);
+			exit (0);
+		case 'd':
+			if (!(arg = poptGetOptArg (ctx)) || (!strcmp (arg, "true") || !strcmp (arg, "yes"))) {
+				daemonize = TRUE;
+				break;
+			}
+		case 'n':
+			daemonize = FALSE;
+			break;
+		case 's':
+			fprintf (stdout, "Managing volumes since the summer of '89\n");
+			break;
+		}
+	}
+	
 	if (daemonize && daemon (0, 0) < 0) {
 		warn ("daemonizing failed: %s", g_strerror (errno));
 		return 1;
 	}
-
-	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-			    argc, argv, GNOME_PARAM_NONE);
-
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	textdomain (PACKAGE);
-
+	
 	client = gnome_master_client ();
-	if (gvm_get_clipboard () && gvm_running_on_console ()) {
+	if (gvm_get_clipboard () && gvm_user_at_console ()) {
 		gnome_client_set_restart_style (client, GNOME_RESTART_ANYWAY);
 	} else {
 		gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-		
-		if (!gvm_running_on_console ())
-			warn ("not on the system console");
-		else
+		if (gvm_user_at_console ())
 			warn ("already running");
 		
 		return 1;
@@ -2346,5 +2556,8 @@ main (int argc, char *argv[])
 	
 	gtk_main ();
 	
+	g_object_unref (program);
+	poptFreeContext (ctx);
+	
 	return 0;
 }
